package cn.leo.nio.utils;

import java.util.concurrent.BlockingQueue;
import java.util.concurrent.Executor;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.SynchronousQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Created by ww on 2016/11/12
 */
public class ThreadPool {
	private static final int CPU_COUNT = Runtime.getRuntime().availableProcessors();
	private static final int CORE_POOL_SIZE = CPU_COUNT + 1;
	private static final int MAXIMUM_POOL_SIZE = CPU_COUNT * 2 + 1;
	private static final int KEEP_ALIVE = 1;

	private static final ThreadFactory sThreadFactory = new ThreadFactory() {
		// ̰߳ȫĵݼӲ
		private final AtomicInteger mCount = new AtomicInteger(1);

		public Thread newThread(Runnable r) {
			return new Thread(r, "Thread #" + mCount.getAndIncrement());
		}
	};

	/**
	 * ̳߳ĵŶӶ,׳쳣
	 */
	private static final BlockingQueue<Runnable> sPoolWorkQueue = new LinkedBlockingQueue<>(128);
	/**
	 * An {@link Executor} that can be used to execute tasks in parallel.
	 */
	public static ThreadPoolExecutor THREAD_POOL_EXECUTOR;

	/**
	 * ִ񣬵̳߳شڹرգᴴµ̳߳
	 */
	public synchronized static void execute(Runnable run) {
		if (run == null) {
			return;
		}
		if (THREAD_POOL_EXECUTOR == null || THREAD_POOL_EXECUTOR.isShutdown()) {
			// ˵
			// ̳߳е߳СmCorePoolSizeֱӴµִ̼̳߳߳
			// ̳߳е߳ĿmCorePoolSizesPoolWorkQueue
			// sPoolWorkQueueеˣᴴµ߳ȥִУ
			// ǵ߳mMaximumPoolSizeʱ׳쳣RejectedExecutionHandler
			// mKeepAliveTimeִ߳Ҷûпִе񣬴ʱ䣬Ĳʱ䵥λ
			// ThreadFactoryÿδµ̹߳

//			THREAD_POOL_EXECUTOR = new ThreadPoolExecutor(CORE_POOL_SIZE, MAXIMUM_POOL_SIZE, KEEP_ALIVE,
//					TimeUnit.SECONDS, sPoolWorkQueue, sThreadFactory);
			// cache
			 THREAD_POOL_EXECUTOR = new ThreadPoolExecutor(0,
			 Integer.MAX_VALUE, 1L, TimeUnit.SECONDS,
			 new SynchronousQueue<Runnable>());
		}
		THREAD_POOL_EXECUTOR.execute(run);
	}

	/**
	 * ȡ̳߳ĳδִе
	 */
	public synchronized static void cancel(Runnable run) {
		if (THREAD_POOL_EXECUTOR != null
				&& (!THREAD_POOL_EXECUTOR.isShutdown() || THREAD_POOL_EXECUTOR.isTerminating())) {
			THREAD_POOL_EXECUTOR.remove(run);
		}
	}

	/**
	 * ̳߳ضǷĳִеĲ
	 */
	public synchronized static boolean contains(Runnable run) {
		if (THREAD_POOL_EXECUTOR != null
				&& (!THREAD_POOL_EXECUTOR.isShutdown() || THREAD_POOL_EXECUTOR.isTerminating())) {
			return THREAD_POOL_EXECUTOR.getQueue().contains(run);
		} else {
			return false;
		}
	}

	/**
	 * ̹ر̳߳أֹͣ񣬰ȴ
	 */
	public synchronized static void stop() {
		if (THREAD_POOL_EXECUTOR != null
				&& (!THREAD_POOL_EXECUTOR.isShutdown() || THREAD_POOL_EXECUTOR.isTerminating())) {
			THREAD_POOL_EXECUTOR.shutdownNow();
		}
	}

	/**
	 * ر̳߳أٽµ񡣵Ѿ񶼽ᱻִϲŹر
	 */
	public synchronized static void shutdown() {
		if (THREAD_POOL_EXECUTOR != null
				&& (!THREAD_POOL_EXECUTOR.isShutdown() || THREAD_POOL_EXECUTOR.isTerminating())) {
			THREAD_POOL_EXECUTOR.shutdown();
		}
	}
}
